#! /usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (C) 2013 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import GLib, GObject, Gio
from gi.repository import Unity, UnityExtras
import gettext
import locale
import os
import random

APP_NAME = 'unity-scope-yelp'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

BUS_NAME = 'com.canonical.Unity.Scope.Help.Yelp'
BUS_PATH = '/com/canonical/unity/scope/help/yelp'

SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
CAT_0_ICON = Gio.ThemedIcon.new(SVG_DIR + 'group-help.svg')
CAT_0_TITLE = _('Official Help Documents')

NO_RESULTS_HINT = _('Sorry, there are no Ubuntu Help results that match your search.')
SEARCH_HINT = _('Ubuntu Help')
SEARCH_URI = ''


class Daemon:
    '''
    Yelp Daemon sets up the lens & searches Ubuntu Help for results
    matching query
    '''

    def __init__(self):
        locale.setlocale(locale.LC_ALL, '')
        self.data = []
        self.setup_yelp()

        self.scope = Unity.Scope.new(BUS_PATH, 'yelp')
        self.scope.props.search_hint = SEARCH_HINT
        self.scope.search_in_global = True
        cats = []
        cats.append(Unity.Category.new('cat_0',
                                       CAT_0_TITLE,
                                       CAT_0_ICON,
                                       Unity.CategoryRenderer.HORIZONTAL_TILE))
        self.scope.props.categories = cats
        self.preferences = Unity.PreferencesManager.get_default()
        self.preferences.connect('notify::remote-content-search', self._on_preference_changed)
        self.scope.connect('search-changed', self.on_search_changed)
        self.scope.export()

    def setup_yelp(self):
        '''
         Parses local help files for <desc> and 1st <title> tags and associates
        them with the page's uri in self.data as: {uri, page description, page title}
        If a help file has no <desc> or <title> tag, it is excluded.
        '''
        from xml.dom import minidom
        mylocale = locale.getlocale()

        data = []
        help_location = "/usr/share/help/" + mylocale[0]
        if not os.path.exists(help_location):
            help_location = "/usr/share/help/" + mylocale[0][:2]
            if not os.path.exists(help_location):
                help_location = "/usr/share/help/C"
        for dirname in os.listdir(help_location):
            directory = help_location + "/" + dirname
            for filename in os.listdir(directory):
                filename = directory + "/" + filename
                if not os.path.isdir(filename):
                    if os.path.isfile(filename):
                        xmldoc = minidom.parse(filename)
                        try:
                            pagenode = xmldoc.getElementsByTagName('page')[0]
                            if pagenode.attributes["type"].value == "topic":
                                desc = ""
                                nodes = xmldoc.getElementsByTagName('desc').item(0).childNodes
                                for node in nodes:
                                    try:
                                        desc += str(node.wholeText)
                                    except:
                                        desc += str(node.childNodes[0].wholeText)
                                desc = desc.strip(' \t\n\r')
                                title = xmldoc.getElementsByTagName('title').item(0).childNodes[0].data
                                if desc == "":
                                    desc = title
                                record = []
                                record.append(filename)
                                record.append(desc)
                                record.append(title)
                                record.append(dirname)
                                data.append(record)
                        except:
                            pass
        random.shuffle(data)
        self.data = data

    def _on_preference_changed(self, *_):
        '''
        Called when a filter is clicked.  Queues a new search
        '''
        self.scope.queue_search_changed(Unity.SearchType.DEFAULT)

    def on_search_changed(self, scope, search, search_type, *_):
        '''
        Called when the search is changed.  Gets the search string and search
        type and passes it to update_results_model()
        Args:
          scope: the scope object
          search: the search string
          search_type: the search type (global or local)
        '''
        model = search.props.results_model
        model.clear()
        if self.preferences.props.remote_content_search != Unity.PreferencesManagerRemoteContent.ALL:
            search.finished()
            return
        search_string = search.props.search_string.strip()
        print 'Search changed to \'%s\'' % search_string
        self.update_results_model(search_string, model)
        search.set_reply_hint('no-results-hint',
                              GLib.Variant.new_string(NO_RESULTS_HINT))
        search.finished()

    def update_results_model(self, search, results):
        '''
        Appends results from yelp to the scope
        '''
        icon_hint1 = Gio.ThemedIcon.new("help").to_string()
        for data in self.data:
            try:
                if data[3] == "ubuntu-help":
                    icon_hint1 = Gio.ThemedIcon.new("distributor-logo").to_string()
                else:
                    icon_hint1 = Gio.ThemedIcon.new(data[3]).to_string()
            except:
                pass

            if search == "":
                if not "/ubuntu-help/" in data[0]:
                    continue

            if not data[1].lower().find(unicode(search.lower(), "utf-8")) == -1:
                results.append(uri=data[0], icon_hint=icon_hint1, category=0, mimetype="text/html", title=data[1], comment="", dnd_uri=data[0], result_type=Unity.ResultType.DEFAULT)
            elif not data[2].lower().find(unicode(search.lower(), "utf-8")) == -1:
                results.append(uri=data[0], icon_hint=icon_hint1, category=0, mimetype="text/html", title=data[2], comment="", dnd_uri=data[0], result_type=Unity.ResultType.DEFAULT)
            elif not data[3].lower().find(unicode(search.lower(), "utf-8")) == -1:
                results.append(uri=data[0], icon_hint=icon_hint1, category=0, mimetype="text/html", title=data[1], comment="", dnd_uri=data[0], result_type=Unity.ResultType.DEFAULT)
            else:
                continue

if __name__ == '__main__':
    daemon = UnityExtras.dbus_own_name(BUS_NAME, Daemon, None)
    if daemon:
        GLib.unix_signal_add_full(0, 2, lambda x: daemon.quit(), None)
        daemon.run([])
